require "ops_script/version"

#
# This module provides some basic features common to many operational unix scripts
#
# * Log file
# * PID file used to insure only one instance is running, handy if running from cron multiple times
# * Options processing with basic options for overriding default log attributes and logfile and pidfile location
#
# == Example
# 
#   require 'ops_script'
#   include OpsScript
#   # Add your own custom options
#   option_parser.on "--bar", "The bar option" do
#     options[:bar] = true
#   end
#   # Run your code
#   OpsScript.run do 
#     puts "BAR" if options[:bar]
#     log.info "My informational message"
#     log.debug "This is a debug message"
#     # do some stuff
#   end
#
# The module provides these options
#    foo.rb -h
#    Usage: foo.rb [options]
#            --debug                      enable debug output
#            --logfile log_file           specify a different log file
#                                         default is ../log/foo.log
#            --log-retention retention    Number of old log files to keep,
#                                         or frequency of rotation (daily, weekly or monthly). Defaults to monthly.
#            --log-size size              Max size in bytes for logfile before rotating.
#                                         Specify number of files to keep with log-retention option.
#            --pidfile pid_file           specify a different pid file
#                                         default is ../tmp/foo.pid
#        -h, -?, --help                   Display this help
#
#
module OpsScript

  require 'date'
  require 'fileutils'
  require 'logger'
  require 'optparse'
  
  def script_basename
    File.basename $0, '.*'
  end
  module_function :script_basename
  
  def options
    OpsScript.options
  end
  
  def self.options
    @options ||= {}
  end
  
  def option_parser
    OpsScript.option_parser
  end
  
  def self.option_parser
    @option_parser ||= OptionParser.new
  end
  
  def process_options
    options[:log_file] = File.expand_path("../../log/#{script_basename}.log", $0)
    options[:pid_file] = File.expand_path("../../tmp/#{script_basename}.pid", $0)
    
    option_parser.on_tail("-h","-?","--help", "Display this help") do
      puts option_parser
      exit
    end
    
    option_parser.on("--debug", "enable debug output") do
      options[:debug] = true
    end
    
    option_parser.on("--logfile log_file", "specify a different log file", "default is #{options[:log_file]}") do |p|
      options[:log_file] = p
    end
    
    option_parser.on("--log-retention retention", "Number of old log files to keep,",
    "or frequency of rotation (daily, weekly or monthly). Defaults to monthly.") do |r|
      options[:log_retention] = r
    end
    
    option_parser.on("--log-size size", "Max size in bytes for logfile before rotating.", 
    "Specify number of files to keep with log-retention option.") do |s|
      options[:log_size] = s
    end
    
    option_parser.on_tail("-h","-?","--help", "Display this help") do
      puts option_parser
      exit
    end
        
    option_parser.on("--pidfile pid_file", "specify a different pid file", "default is #{options[:pid_file]}") do |p|
      options[:pid_file] = p
    end
    
    option_parser.parse!
  end
  private :process_options
  
  def create_directories
    FileUtils.mkdir_p File.dirname(File.expand_path(options[:log_file]))
    FileUtils.mkdir_p File.dirname(File.expand_path(options[:pid_file]))
  end
  private :create_directories
  
  def already_running?
    File.exists?(options[:pid_file])
  end
  private :already_running?
  
  def create_pid
    ::File.open(options[:pid_file], 'w' ){ |f| f.write("#{Process.pid}")}
  end
  private :create_pid
  
  def remove_pid
    File.unlink(options[:pid_file])
  end
  private :remove_pid
  
  # Use log in your script to log messages
  def log
    OpsScript.log
  end
  
  def self.log
    @log ||= Logger.new(options[:log_file], options[:log_retention] || 'monthly', options[:log_size])
  end
  
  def script
    File.basename $0
  end
  private :script
  
  def self.run
    process_options
    if already_running?
      puts "Found #{options[:pid_file]} file. Must be running already. Exiting."
      exit
    else
      create_directories
      create_pid
      log.level = Logger::INFO unless options[:debug]
      log.info "--"
      log.info "#{script} starting"
    end
    begin
      if block_given?
        yield
      else
        raise "No block given. Nothing to run."
      end
    rescue => e
      log.error e.message
      log.debug e.backtrace.join('\n')
      raise
    ensure
      log.info "#{script} exiting"
      log.close
      remove_pid
    end
  end
end
